/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2016 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

var juju = { components: {} }; // eslint-disable-line no-unused-vars

chai.config.includeStack = true;
chai.config.truncateThreshold = 0;

/**
  Convenience function for quickly and easily creating a DeploymentFlow
  component. It accept a props param, which can be used to change defaults (on
  required props) or provide values (on optional props).
 */
const createDeploymentFlow = (props = {}) => {
  // Setup all the default props.
  const appId = 'service1';
  const charmsGetById = sinon.stub();
  charmsGetById.withArgs('service1').returns({
    get: sinon.stub().withArgs('terms').returns(['service1-terms'])
  });
  charmsGetById.withArgs('mysql').returns({
    get: sinon.stub().withArgs('terms').returns(['my-terms', 'general-terms'])
  });
  const getAgreementsByTerms = sinon.stub().callsArgWith(1, null, [{
    name: 'service1-terms',
    content: 'service1 terms.',
    owner: 'spinach',
    revision: 5
  }, {
    name: 'my-terms',
    content: 'Mysql terms.',
    revision: 9
  }]);
  const groupedChanges = {
    _deploy: {
      appId: {
        command: {
          args: [{ charmURL: appId }]
        }
      }
    },
    _addMachines: { machine: 'machine1' }
  };
  // Note that the defaults are *only* set for required DeploymentFlow props.
  const defaults = {
    acl: { isReadOnly: sinon.stub().returns(false) },
    addAgreement: sinon.stub(),
    addNotification: sinon.stub(),
    applications: [],
    changeState: sinon.stub(),
    changes: {},
    changesFilterByParent: sinon.stub(),
    charmsGetById: charmsGetById,
    charmstore: {},
    controllerIsReady: sinon.stub(),
    createToken: sinon.stub(),
    createUser: sinon.stub(),
    ddData: {},
    deploy: sinon.stub().callsArg(0),
    formatConstraints: sinon.stub(),
    generateAllChangeDescriptions: sinon.stub(),
    generateCloudCredentialName: sinon.stub(),
    generateMachineDetails: sinon.stub(),
    getAgreementsByTerms: getAgreementsByTerms,
    getCloudCredentialNames: sinon.stub(),
    getCloudCredentials: sinon.stub(),
    getCloudProviderDetails: sinon.stub(),
    getCountries: sinon.stub(),
    getCurrentChangeSet: sinon.stub(),
    getDiagramURL: sinon.stub(),
    getUser: sinon.stub(),
    getUserName: sinon.stub().returns('dalek'),
    groupedChanges: groupedChanges,
    isLoggedIn: sinon.stub().returns(true),
    listBudgets: sinon.stub(),
    listClouds: sinon.stub(),
    listPlansForCharm: sinon.stub(),
    loginToController: sinon.stub(),
    makeEntityModel: sinon.stub(),
    modelName: 'Pavlova',
    profileUsername: 'Spinach',
    renderMarkdown: sinon.stub(),
    sendAnalytics: sinon.stub(),
    servicesGetById: sinon.stub(),
    setModelName: sinon.stub(),
    showPay: false,
    showTerms: sinon.stub(),
    updateCloudCredential: sinon.stub(),
    validateForm: sinon.stub(),
    withPlans: true
  };
  // Merge the user-specified props with the default props.
  const userKeys = Object.keys(props);
  Object.keys(defaults).forEach(defaultKey => {
    // If a certain prop isn't specified by the user, assign it to the
    // default value.
    if (userKeys.indexOf(defaultKey) < 0) {
      props[defaultKey] = defaults[defaultKey];
    }
  });
  return jsTestUtils.shallowRender(React.createElement(
    juju.components.DeploymentFlow,
    props,
    React.createElement(
      'span',
      null,
      'content'
    )
  ), true);
};

describe('DeploymentFlow', function () {
  let applications;

  beforeAll(function (done) {
    // By loading this file it adds the component to the juju components.
    YUI().use('deployment-flow', function () {
      done();
    });
  });

  beforeEach(() => {
    applications = [{ get: sinon.stub().returns('service1') }, { get: sinon.stub().returns('mysql') }, { get: sinon.stub().returns('service1') }];
  });

  it('can render', function () {
    const formatConstraints = sinon.stub();
    const generateMachineDetails = sinon.stub();
    const validateForm = sinon.stub();
    const renderer = createDeploymentFlow({
      formatConstraints: formatConstraints,
      generateMachineDetails: generateMachineDetails,
      getAgreementsByTerms: sinon.stub().callsArgWith(1, null, []),
      modelCommitted: false,
      validateForm: validateForm,
      withPlans: true
    });
    const instance = renderer.getMountedInstance();
    const output = renderer.getRenderOutput();
    const props = instance.props;
    const expected = React.createElement(
      juju.components.DeploymentPanel,
      {
        changeState: props.changeState,
        title: 'Pavlova',
        sendAnalytics: sinon.stub() },
      React.createElement(
        juju.components.DeploymentSection,
        {
          instance: 'deployment-model-name',
          showCheck: false,
          title: 'Set your model name' },
        React.createElement(
          'div',
          { className: 'six-col' },
          React.createElement(juju.components.GenericInput, {
            disabled: false,
            key: 'modelName',
            label: 'Model name',
            required: true,
            onBlur: instance._updateModelName,
            ref: 'modelName',
            validate: [{
              regex: /\S+/,
              error: 'This field is required.'
            }, {
              regex: /^([a-z0-9]([a-z0-9-]*[a-z0-9])?)?$/,
              error: 'This field must only contain lowercase ' + 'letters, numbers, and hyphens. It must not start ' + 'or end with a hyphen.'
            }],
            value: 'Pavlova' })
        )
      ),
      React.createElement(
        juju.components.DeploymentSection,
        {
          buttons: undefined,
          completed: false,
          disabled: false,
          instance: 'deployment-cloud',
          showCheck: true,
          title: 'Choose cloud to deploy to' },
        React.createElement(juju.components.DeploymentCloud, {
          acl: props.acl,
          cloud: null,
          controllerIsReady: props.controllerIsReady,
          listClouds: props.listClouds,
          getCloudProviderDetails: props.getCloudProviderDetails,
          setCloud: instance._setCloud })
      ),
      React.createElement(
        juju.components.DeploymentSection,
        {
          completed: false,
          disabled: true,
          instance: 'deployment-ssh-key',
          showCheck: false },
        React.createElement(juju.components.DeploymentSSHKey, {
          cloud: null,
          setSSHKey: instance._setSSHKey
        })
      ),
      undefined,
      React.createElement(
        juju.components.DeploymentSection,
        {
          completed: false,
          disabled: true,
          instance: 'deployment-machines',
          showCheck: false,
          title: 'Machines to be deployed' },
        React.createElement(juju.components.DeploymentMachines, {
          acl: props.acl,
          cloud: null,
          formatConstraints: formatConstraints,
          generateMachineDetails: generateMachineDetails,
          machines: props.groupedChanges._addMachines })
      ),
      React.createElement(
        juju.components.DeploymentSection,
        {
          completed: false,
          disabled: true,
          instance: 'deployment-services',
          showCheck: true,
          title: React.createElement(
            'span',
            { className: 'deployment-flow__service-title' },
            'Applications to be deployed',
            React.createElement(juju.components.GenericButton, {
              action: instance._toggleChangelogs,
              type: 'inline-neutral',
              extraClasses: 'right',
              title: 'Show changelog' })
          ) },
        React.createElement(juju.components.DeploymentServices, {
          acl: props.acl,
          changesFilterByParent: props.changesFilterByParent,
          charmsGetById: props.charmsGetById,
          generateAllChangeDescriptions: props.generateAllChangeDescriptions,
          groupedChanges: props.groupedChanges,
          listPlansForCharm: props.listPlansForCharm,
          parseTermId: instance._parseTermId,
          servicesGetById: props.servicesGetById,
          showChangelogs: false,
          showTerms: props.showTerms,
          withPlans: true })
      ),
      React.createElement(
        juju.components.DeploymentSection,
        {
          completed: false,
          disabled: true,
          instance: 'deployment-budget',
          showCheck: true,
          title: 'Confirm budget' },
        React.createElement(juju.components.DeploymentBudget, {
          acl: props.acl,
          listBudgets: props.listBudgets,
          setBudget: instance._setBudget,
          user: 'dalek' })
      ),
      React.createElement(
        juju.components.DeploymentSection,
        {
          completed: false,
          disabled: true,
          instance: 'deployment-changes',
          showCheck: false,
          title: 'Model changes' },
        React.createElement(juju.components.DeploymentChanges, {
          getCurrentChangeSet: props.getCurrentChangeSet,
          generateAllChangeDescriptions: props.generateAllChangeDescriptions })
      ),
      null,
      React.createElement(
        'div',
        { className: 'twelve-col' },
        React.createElement(
          'div',
          { className: 'deployment-flow__deploy' },
          undefined,
          React.createElement(
            'div',
            { className: 'deployment-flow__deploy-action' },
            React.createElement(juju.components.GenericButton, {
              action: instance._handleDeploy,
              disabled: true,
              type: 'positive',
              title: 'Deploy' })
          )
        )
      )
    );
    expect(output).toEqualJSX(expected);
  });

  it('renders direct deploy when ddData is set', () => {
    const getEntity = sinon.stub();
    const makeEntityModel = sinon.stub();
    const renderMarkdown = sinon.stub();
    const renderer = createDeploymentFlow({
      ddData: { id: 'cs:bundles/kubernetes-core-8' },
      getEntity: getEntity,
      makeEntityModel: makeEntityModel,
      modelCommitted: false,
      renderMarkdown: renderMarkdown
    });
    const output = renderer.getRenderOutput();
    const instance = renderer.getMountedInstance();
    expect(output.props.children[0]).toEqualJSX(React.createElement(juju.components.DeploymentDirectDeploy, {
      ddData: { id: 'cs:bundles/kubernetes-core-8' },
      getDiagramURL: instance.props.getDiagramURL,
      getEntity: getEntity,
      makeEntityModel: makeEntityModel,
      renderMarkdown: renderMarkdown
    }));
  });

  it('can display the cloud section as complete', function () {
    const renderer = createDeploymentFlow({
      credential: 'cred',
      modelCommitted: false
    });
    const instance = renderer.getMountedInstance();
    instance.setState({ cloud: { name: 'cloud' } });
    const output = renderer.getRenderOutput();
    assert.strictEqual(output.props.children[2].props.completed, true);
  });

  it('does not display the cloud section if complete + committed', function () {
    const renderer = createDeploymentFlow({
      cloud: { name: 'cloud' },
      credential: 'cred',
      modelCommitted: true
    });
    const output = renderer.getRenderOutput();
    assert.strictEqual(output.props.children[1], undefined);
  });

  it('does not show the model name when comitting', function () {
    const renderer = createDeploymentFlow({
      modelCommitted: true
    });
    const output = renderer.getRenderOutput();
    assert.isUndefined(output.props.children[1]);
  });

  it('correctly sets the cloud title if no cloud is chosen', function () {
    const renderer = createDeploymentFlow();
    const output = renderer.getRenderOutput();
    assert.equal(output.props.children[2].props.title, 'Choose cloud to deploy to');
  });

  it('can clear the cloud and credential when changing clouds', function () {
    const renderer = createDeploymentFlow({
      cloud: { name: 'cloud-1' },
      credential: 'cloud-1-cred'
    });
    const instance = renderer.getMountedInstance();
    instance._setCloud({ name: 'cloud-2' });
    instance._setCredential('cloud-2-cred');
    const output = renderer.getRenderOutput();
    assert.isNotNull(instance.state.cloud);
    assert.isNotNull(instance.state.credential);
    output.props.children[2].props.buttons[0].action();
    assert.isNull(instance.state.cloud);
    assert.isNull(instance.state.credential);
    assert.equal(instance.props.sendAnalytics.callCount, 1);
    assert.deepEqual(instance.props.sendAnalytics.args[0], ['Deployment Flow', 'Deployment started', 'is DD - is new model - doesn\'t have USSO']);
  });

  it('can enable the credential section', function () {
    const renderer = createDeploymentFlow({
      modelCommitted: false
    });
    const instance = renderer.getMountedInstance();
    instance.setState({ cloud: { name: 'cloud' } });
    const output = renderer.getRenderOutput();
    assert.strictEqual(output.props.children[3].props.disabled, false);
  });

  it('can hide the credential section', function () {
    const renderer = createDeploymentFlow({
      modelCommitted: true
    });
    const instance = renderer.getMountedInstance();
    instance.setState({ cloud: { name: 'cloud' } });
    const output = renderer.getRenderOutput();
    assert.strictEqual(output.props.children[3], undefined);
  });

  it('can enable the machines section', function () {
    const renderer = createDeploymentFlow({
      cloud: { name: 'cloud' },
      credential: 'cred',
      modelCommitted: true
    });
    const output = renderer.getRenderOutput();
    assert.strictEqual(output.props.children[6].props.disabled, false);
  });

  it('can enable the services section', function () {
    const renderer = createDeploymentFlow({
      cloud: { name: 'cloud' },
      credential: 'cred',
      modelCommitted: true
    });
    const output = renderer.getRenderOutput();
    assert.strictEqual(output.props.children[6].props.disabled, false);
  });

  it('displays the VPC section in new AWS models', function () {
    const renderer = createDeploymentFlow({ modelCommitted: false });
    const instance = renderer.getMountedInstance();
    instance._setCloud({ name: 'aws' });
    const output = renderer.getRenderOutput();
    const expectedOutput = React.createElement(
      juju.components.DeploymentSection,
      {
        completed: false,
        disabled: false,
        instance: 'deployment-vpc',
        showCheck: false },
      React.createElement(juju.components.DeploymentVPC, { setVPCId: instance._setVPCId })
    );
    assert.deepEqual(output.props.children[5], expectedOutput);
  });

  it('can enable the budget section', function () {
    const renderer = createDeploymentFlow({
      cloud: { name: 'cloud' },
      credential: 'cred',
      modelCommitted: true,
      withPlans: true
    });
    const output = renderer.getRenderOutput();
    assert.isFalse(output.props.children[6].props.disabled);
  });

  it('can show the payments section', function () {
    const acl = { isReadOnly: sinon.stub() };
    const addNotification = sinon.stub();
    const createToken = sinon.stub();
    const createUser = sinon.stub();
    const getCountries = sinon.stub();
    const getUser = sinon.stub();
    const validateForm = sinon.stub();
    const createCardElement = sinon.stub();
    const renderer = createDeploymentFlow({
      acl: acl,
      addNotification: addNotification,
      cloud: { name: 'cloud' },
      createCardElement: createCardElement,
      createToken: createToken,
      createUser: createUser,
      credential: 'cred',
      getCountries: getCountries,
      getUser: getUser,
      isLegacyJuju: false,
      profileUsername: 'spinach',
      showPay: true,
      validateForm: validateForm
    });
    const instance = renderer.getMountedInstance();
    const output = renderer.getRenderOutput();
    const expected = React.createElement(
      juju.components.DeploymentSection,
      {
        completed: false,
        disabled: false,
        instance: 'deployment-payment',
        showCheck: true,
        title: 'Payment details' },
      React.createElement(juju.components.DeploymentPayment, {
        acl: acl,
        addNotification: addNotification,
        createCardElement: createCardElement,
        createToken: createToken,
        createUser: createUser,
        getCountries: getCountries,
        getUser: getUser,
        paymentUser: null,
        setPaymentUser: instance._setPaymentUser,
        username: 'spinach',
        validateForm: validateForm })
    );
    assert.deepEqual(output.props.children[10], expected);
  });

  it('can hide the agreements section', function () {
    const renderer = createDeploymentFlow({
      getAgreementsByTerms: sinon.stub().callsArgWith(1, null, [])
    });
    const output = renderer.getRenderOutput();
    assert.isUndefined(output.props.children[11].props.children.props.children[0]);
  });

  it('can handle the agreements when there are no added apps', function () {
    const renderer = createDeploymentFlow({
      cloud: { name: 'cloud' },
      credential: 'cred',
      groupedChanges: { _addMachines: { machine: 'machine1' } },
      modelCommitted: true
    });
    const output = renderer.getRenderOutput();
    assert.isUndefined(output.props.children[11].props.children.props.children[0]);
  });

  it('can display the agreements section', function () {
    const charmsGetById = sinon.stub().returns({
      get: sinon.stub().withArgs('terms').returns(['django-terms'])
    });
    const renderer = createDeploymentFlow({
      cloud: { name: 'cloud' },
      credential: 'cred',
      charmsGetById: charmsGetById,
      modelCommitted: true
    });
    const output = renderer.getRenderOutput();
    const instance = renderer.getMountedInstance();
    const agreements = output.props.children[11].props.children.props.children[0];
    const expected = React.createElement(
      'div',
      { className: 'deployment-flow__deploy-option' },
      React.createElement('input', { className: 'deployment-flow__deploy-checkbox',
        onChange: instance._handleTermsAgreement,
        disabled: false,
        id: 'terms',
        type: 'checkbox' }),
      React.createElement(
        'label',
        { className: 'deployment-flow__deploy-label',
          htmlFor: 'terms' },
        'I agree to all terms.'
      )
    );
    assert.deepEqual(agreements, expected);
  });

  it('can disable the agreements section', function () {
    const renderer = createDeploymentFlow({
      modelCommitted: false
    });
    const output = renderer.getRenderOutput();
    const agreements = output.props.children[11].props.children.props.children[0];
    const className = agreements.props.className;
    const expectedClass = 'deployment-flow__deploy-option--disabled';
    assert.isTrue(className.indexOf(expectedClass) > 0);
    assert.isTrue(agreements.props.children[0].props.disabled);
  });

  it('renders the login when necessary', function () {
    const renderer = createDeploymentFlow({
      isLoggedIn: sinon.stub().returns(false),
      loginToController: sinon.stub(),
      modelCommitted: true
    });
    const output = renderer.getRenderOutput();
    const instance = renderer.getMountedInstance();
    const loginLink = output.props.children[1].props.children.props.children[2].props.children;
    const expected = React.createElement(
      juju.components.DeploymentSection,
      {
        instance: 'deployment-model-login',
        showCheck: true,
        title: 'You\'re almost ready to deploy!' },
      React.createElement(
        'div',
        { className: 'twelve-col' },
        React.createElement(
          'p',
          { className: 'deployment-login__intro' },
          'You will need to sign in with an Ubuntu One account to deploy your model with Juju-as-a-Service.'
        ),
        React.createElement(
          'div',
          { className: 'deployment-login__features' },
          React.createElement(
            'div',
            { className: 'six-col' },
            React.createElement(
              'div',
              { className: 'deployment-login__feature' },
              React.createElement(juju.components.SvgIcon, { name: 'task-done_16', size: '16' }),
              'Deploy to all major clouds directly from your browser.'
            ),
            React.createElement(
              'div',
              { className: 'deployment-login__feature' },
              React.createElement(juju.components.SvgIcon, { name: 'task-done_16', size: '16' }),
              'Identity management across all models.'
            )
          ),
          React.createElement(
            'div',
            { className: 'six-col last-col' },
            React.createElement(
              'div',
              { className: 'deployment-login__feature' },
              React.createElement(juju.components.SvgIcon, { name: 'task-done_16', size: '16' }),
              'Hosted and managed juju controllers.'
            ),
            React.createElement(
              'div',
              { className: 'deployment-login__feature' },
              React.createElement(juju.components.SvgIcon, { name: 'task-done_16', size: '16' }),
              'Reusable shareable models with unlimited users.'
            )
          )
        ),
        React.createElement(
          'div',
          { className: 'deployment-login__login' },
          React.createElement(
            juju.components.USSOLoginLink,
            {
              gisf: instance.props.gisf,
              callback: loginLink.props.callback,
              displayType: 'button',
              loginToController: instance.props.loginToController },
            'Login'
          )
        ),
        React.createElement(
          'div',
          { className: 'deployment-login__signup' },
          'Do not have an account?',
          React.createElement(
            juju.components.USSOLoginLink,
            {
              gisf: instance.props.gisf,
              callback: loginLink.props.callback,
              displayType: 'text',
              loginToController: instance.props.loginToController },
            'Sign up'
          )
        )
      )
    );
    expect(output.props.children[1]).toEqualJSX(expected);
  });

  // Click log in and pass the given error string to the login callback used by
  // the component. Return the component instance.
  const login = function (err) {
    const renderer = createDeploymentFlow({
      isLoggedIn: sinon.stub().returns(false),
      loginToController: sinon.stub(),
      modelCommitted: true
    });
    const instance = renderer.getMountedInstance();
    assert.strictEqual(instance.state.loggedIn, false);
    instance.refs = {
      modelName: {
        getValue: sinon.stub().returns('Lamington')
      }
    };
    const output = renderer.getRenderOutput();
    const loginSection = output.props.children[1].props.children;
    const loginButton = loginSection.props.children[2].props.children;
    const loginToController = instance.props.loginToController;
    // Call the supplied callback function which is called after the user
    // logs in.
    loginButton.props.loginToController(loginButton.props.callback);
    assert.strictEqual(loginToController.callCount, 1);
    const cb = loginToController.args[0][0];
    cb(err);
    return instance;
  };

  it('can login (success)', function () {
    const instance = login(null);
    assert.strictEqual(instance.state.loggedIn, true);
  });

  it('can login (failure)', function () {
    const instance = login('bad wolf');
    assert.strictEqual(instance.state.loggedIn, false);
  });

  it('can deploy', function () {
    const charmsGetById = sinon.stub().withArgs('service1').returns({
      get: sinon.stub().withArgs('terms').returns([])
    });
    const renderer = createDeploymentFlow({
      charmsGetById: charmsGetById,
      cloud: { name: 'cloud' },
      credential: 'cred',
      modelCommitted: true,
      region: 'north'
    });
    const instance = renderer.getMountedInstance();
    instance.refs = {
      modelName: {
        getValue: sinon.stub().returns('Lamington')
      }
    };
    instance._updateModelName();
    const props = instance.props;
    const output = renderer.getRenderOutput();
    output.props.children[11].props.children.props.children[1].props.children.props.action();
    assert.equal(props.deploy.callCount, 1);
    assert.strictEqual(props.deploy.args[0].length, 4);
    assert.equal(props.deploy.args[0][2], 'Lamington');
    assert.deepEqual(props.deploy.args[0][3], {
      config: {},
      cloud: 'cloud',
      credential: 'cred',
      region: 'north'
    });
    assert.equal(props.changeState.callCount, 1);
    assert.equal(props.setModelName.callCount, 1);
    assert.equal(props.setModelName.args[0][0], 'Lamington');
    assert.equal(props.sendAnalytics.callCount, 2);
    assert.deepEqual(props.sendAnalytics.args[0], ['Deployment Flow', 'Deployment started', 'is DD - is model update - doesn\'t have USSO']);
    assert.deepEqual(props.sendAnalytics.args[1], ['Deployment Flow', 'Button click', 'Deploy model - is DD' + ' - is model update - doesn\'t have USSO']);
  });

  it('can agree to terms during deploy', function () {
    const renderer = createDeploymentFlow({
      cloud: { name: 'cloud' },
      credential: 'cred',
      modelCommitted: true,
      region: 'north'
    });
    const instance = renderer.getMountedInstance();
    instance.refs = {
      modelName: {
        getValue: sinon.stub().returns('Lamington')
      }
    };
    instance._updateModelName();
    instance._handleTermsAgreement({ target: { checked: true } });
    const props = instance.props;
    const output = renderer.getRenderOutput();
    output.props.children[11].props.children.props.children[1].props.children.props.action();
    assert.equal(props.deploy.callCount, 0, 'The deploy function should not be called');
    assert.equal(props.addAgreement.callCount, 1, 'The addAgreement function was not called');
    assert.deepEqual(props.addAgreement.args[0][0], [{
      name: 'service1-terms',
      owner: 'spinach',
      revision: 5
    }, {
      name: 'my-terms',
      revision: 9
    }], 'The agreement passed in was not as expected.');
    assert.equal(props.setModelName.callCount, 1, 'The setModelName function was not called.');
    assert.equal(props.setModelName.args[0][0], 'Lamington', 'The setModelName function was not called with the right model name');
  });

  it('allows or disallows deployments', function () {
    const tests = [{
      about: 'no model name',
      state: { modelName: '' },
      allowed: false
    }, {
      about: 'no cloud',
      state: { modelName: 'mymodel' },
      modelCommitted: true,
      allowed: false
    }, {
      about: 'read only',
      state: {
        modelName: 'mymodel',
        cloud: { cloudType: 'aws' }
      },
      isReadOnly: true,
      modelCommitted: true,
      allowed: false
    }, {
      about: 'deploying',
      state: {
        modelName: 'mymodel',
        cloud: { cloudType: 'aws' },
        deploying: true
      },
      modelCommitted: true,
      allowed: false
    }, {
      about: 'model committed',
      state: {
        modelName: 'mymodel',
        cloud: { cloudType: 'aws' },
        credential: 'cred'
      },
      modelCommitted: true,
      noTerms: true,
      allowed: true
    }, {
      about: 'no ssh on azure',
      state: {
        modelName: 'mymodel',
        cloud: { cloudType: 'azure' },
        credential: 'cred'
      },
      allowed: false
    }, {
      about: 'no ssh on aws',
      state: {
        modelName: 'mymodel',
        cloud: { cloudType: 'aws' },
        credential: 'cred'
      },
      noTerms: true,
      allowed: true
    }, {
      about: 'ssh provided on azure',
      state: {
        modelName: 'mymodel',
        cloud: { cloudType: 'azure' },
        credential: 'cred',
        sshKey: 'mykey'
      },
      noTerms: true,
      allowed: true
    }, {
      about: 'ssh provided on aws',
      state: {
        modelName: 'mymodel',
        cloud: { cloudType: 'aws' },
        credential: 'cred',
        sshKey: 'mykey'
      },
      noTerms: true,
      allowed: true
    }, {
      about: 'no payment user',
      state: {
        modelName: 'mymodel',
        cloud: { cloudType: 'aws' },
        credential: 'cred',
        paymentUser: null,
        sshKey: 'mykey'
      },
      noTerms: true,
      showPay: true,
      allowed: false
    }, {
      about: 'terms not agreed',
      state: {
        modelName: 'mymodel',
        cloud: { cloudType: 'aws' },
        credential: 'cred',
        terms: ['foo'],
        termsAgreed: false
      },
      allowed: false,
      includeAgreements: true
    }, {
      about: 'terms agreed',
      state: {
        modelName: 'mymodel',
        cloud: { cloudType: 'aws' },
        credential: 'cred',
        terms: ['foo'],
        termsAgreed: true
      },
      noTerms: true,
      allowed: true
    }, {
      about: 'terms not finished loading',
      state: {
        loadingTerms: true,
        modelName: 'mymodel',
        cloud: { cloudType: 'aws' },
        credential: 'cred',
        terms: ['foo'],
        termsAgreed: true
      },
      allowed: false
    }, {
      about: 'deploy should be disabled when there no credentials',
      state: {
        modelName: 'mymodel',
        cloud: { cloudType: 'aws' },
        credential: null
      },
      allowed: false
    }];
    tests.forEach(test => {
      const charmsGetById = sinon.stub();
      if (test.noTerms) {
        charmsGetById.withArgs('service1').returns({
          get: sinon.stub().returns([])
        });
      } else {
        charmsGetById.withArgs('service1').returns({
          get: sinon.stub().returns(['service1-terms'])
        });
      }
      const props = {
        acl: { isReadOnly: () => !!test.isReadOnly },
        applications: applications,
        charmsGetById: charmsGetById,
        modelCommitted: !!test.modelCommitted,
        showPay: test.showPay || false
      };
      if (!test.includeAgreements) {
        props.getAgreementsByTerms = sinon.stub();
      }
      const renderer = createDeploymentFlow(props);
      const instance = renderer.getMountedInstance();
      instance.setState(test.state);
      const allowed = instance._deploymentAllowed();
      assert.strictEqual(allowed, test.allowed, test.about);
    });
  });

  it('can disable the deploy button on deploy', function () {
    const charmsGetById = sinon.stub().withArgs('service1').returns({
      get: sinon.stub().withArgs('terms').returns([])
    });
    const renderer = createDeploymentFlow({
      charmsGetById: charmsGetById,
      cloud: { name: 'cloud' },
      credential: 'cred',
      deploy: sinon.stub(), // Don't trigger a re-render by calling callback.
      modelCommitted: true,
      region: 'north'
    });
    const instance = renderer.getMountedInstance();
    instance.refs = {
      modelName: {
        getValue: sinon.stub().returns('Lamington')
      }
    };
    let output = renderer.getRenderOutput();
    let deployButton = output.props.children[11].props.children.props.children[1].props.children;
    deployButton.props.action();

    // .action() rerenders the component so we need to get it again
    output = renderer.getRenderOutput();
    deployButton = output.props.children[11].props.children.props.children[1].props.children;

    assert.equal(deployButton.props.disabled, true);
    assert.equal(deployButton.props.title, 'Deploying...');
  });

  it('can deploy without updating the model name', function () {
    const charmsGetById = sinon.stub().withArgs('service1').returns({
      get: sinon.stub().withArgs('terms').returns([])
    });
    const renderer = createDeploymentFlow({
      charmsGetById: charmsGetById,
      cloud: { name: 'cloud' },
      credential: 'cred',
      modelCommitted: true,
      region: 'north'
    });
    const instance = renderer.getMountedInstance();
    instance.refs = {};
    const output = renderer.getRenderOutput();
    output.props.children[11].props.children.props.children[1].props.children.props.action();
    const deploy = instance.props.deploy;
    assert.equal(deploy.callCount, 1);
    assert.strictEqual(deploy.args[0].length, 4);
    assert.equal(deploy.args[0][2], 'Pavlova');
    assert.deepEqual(deploy.args[0][3], {
      config: {},
      cloud: 'cloud',
      credential: 'cred',
      region: 'north'
    });
    assert.equal(instance.props.changeState.callCount, 1);
  });

  it('can deploy with SSH keys', function () {
    const charmsGetById = sinon.stub().withArgs('service1').returns({
      get: sinon.stub().withArgs('terms').returns([])
    });
    const renderer = createDeploymentFlow({
      charmsGetById: charmsGetById,
      cloud: { name: 'azure' },
      credential: 'creds',
      modelCommitted: true,
      modelName: 'mymodel',
      region: 'skaro'
    });
    const instance = renderer.getMountedInstance();
    instance._setSSHKey('my SSH key');
    const output = renderer.getRenderOutput();
    output.props.children[11].props.children.props.children[1].props.children.props.action();
    const deploy = instance.props.deploy;
    assert.equal(deploy.callCount, 1);
    assert.strictEqual(deploy.args[0].length, 4);
    assert.equal(deploy.args[0][2], 'mymodel');
    assert.deepEqual(deploy.args[0][3], {
      credential: 'creds',
      cloud: 'azure',
      region: 'skaro',
      config: { 'authorized-keys': 'my SSH key' }
    });
    assert.equal(instance.props.changeState.callCount, 1);
  });

  it('can deploy with a VPC id', function () {
    const charmsGetById = sinon.stub().withArgs('service1').returns({
      get: sinon.stub().withArgs('terms').returns([])
    });
    const renderer = createDeploymentFlow({
      charmsGetById: charmsGetById,
      cloud: { name: 'aws' },
      credential: 'creds',
      modelCommitted: true,
      modelName: 'mymodel',
      region: 'skaro'
    });
    const instance = renderer.getMountedInstance();
    instance._setVPCId('my VPC id');
    const output = renderer.getRenderOutput();
    output.props.children[11].props.children.props.children[1].props.children.props.action();
    const deploy = instance.props.deploy;
    assert.equal(deploy.callCount, 1);
    assert.strictEqual(deploy.args[0].length, 4);
    assert.equal(deploy.args[0][2], 'mymodel');
    assert.deepEqual(deploy.args[0][3], {
      credential: 'creds',
      cloud: 'aws',
      region: 'skaro',
      config: { 'vpc-id': 'my VPC id', 'vpc-id-force': false }
    });
    assert.equal(instance.props.changeState.callCount, 1);
  });

  it('can deploy with a forced VPC id', function () {
    const charmsGetById = sinon.stub().withArgs('service1').returns({
      get: sinon.stub().withArgs('terms').returns([])
    });
    const renderer = createDeploymentFlow({
      charmsGetById: charmsGetById,
      cloud: { name: 'aws' },
      credential: 'creds',
      modelCommitted: true,
      modelName: 'mymodel',
      region: 'skaro'
    });
    const instance = renderer.getMountedInstance();
    instance._setVPCId('my VPC id', true);
    const output = renderer.getRenderOutput();
    output.props.children[11].props.children.props.children[1].props.children.props.action();
    const deploy = instance.props.deploy;
    assert.equal(deploy.callCount, 1);
    assert.strictEqual(deploy.args[0].length, 4);
    assert.equal(deploy.args[0][2], 'mymodel');
    assert.deepEqual(deploy.args[0][3], {
      credential: 'creds',
      cloud: 'aws',
      region: 'skaro',
      config: { 'vpc-id': 'my VPC id', 'vpc-id-force': true }
    });
    assert.equal(instance.props.changeState.callCount, 1);
  });

  it('focuses on the model name field when loaded', function () {
    const renderer = createDeploymentFlow();
    const instance = renderer.getMountedInstance();
    instance.refs = { modelName: { focus: sinon.stub() } };
    instance.componentDidMount();
    assert.equal(instance.refs.modelName.focus.callCount, 1);
  });

  it('aborts the requests when unmounting', function () {
    const abort = sinon.stub();
    const renderer = createDeploymentFlow({
      applications: applications,
      getAgreementsByTerms: sinon.stub().returns({ abort: abort })
    });
    renderer.unmount();
    assert.deepEqual(abort.callCount, 1);
  });

  describe('_parseTermId', function () {
    let parseTermId;

    beforeAll(function () {
      parseTermId = juju.components.DeploymentFlow.prototype._parseTermId;
    });

    it('can get the name', function () {
      assert.deepEqual(parseTermId('general'), {
        name: 'general',
        owner: null,
        revision: null
      });
    });

    it('can get the name and owner', function () {
      assert.deepEqual(parseTermId('spinach/general'), {
        name: 'general',
        owner: 'spinach',
        revision: null
      });
    });

    it('can get the name and revision', function () {
      assert.deepEqual(parseTermId('general/22'), {
        name: 'general',
        owner: null,
        revision: 22
      });
    });

    it('can get the name, owner and revision', function () {
      assert.deepEqual(parseTermId('spinach/general/22'), {
        name: 'general',
        owner: 'spinach',
        revision: 22
      });
    });
  });
});