/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2015 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

var juju = { components: {} }; // eslint-disable-line no-unused-vars
var testUtils = React.addons.TestUtils;

chai.config.includeStack = true;
chai.config.truncateThreshold = 0;

describe('EntityHeader', function () {
  let acl, mockEntity, urllib;

  beforeAll(function (done) {
    // By loading these files it makes their classes available in the tests.
    YUI().use('entity-header', function () {
      done();
    });
  });

  beforeEach(function () {
    acl = { isReadOnly: sinon.stub().returns(false) };
    mockEntity = jsTestUtils.makeEntity();
    urllib = sinon.stub();
    urllib.fromLegacyString = sinon.stub().returns({
      revision: 42,
      path: sinon.stub().returns('u/who/django/42')
    });
  });

  afterEach(function () {
    mockEntity = undefined;
  });

  it('renders the latest entity properly', function () {
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.EntityHeader, {
      acl: acl,
      addNotification: sinon.stub(),
      deployService: sinon.spy(),
      changeState: sinon.spy(),
      entityModel: mockEntity,
      getModelName: sinon.stub(),
      getBundleYAML: sinon.stub(),
      hasPlans: false,
      importBundleYAML: sinon.stub(),
      pluralize: sinon.stub(),
      scrollPosition: 0,
      urllib: urllib
    }), true);
    const instance = renderer.getMountedInstance();
    const output = renderer.getRenderOutput();
    const expectedOutput = React.createElement(
      'div',
      { className: 'row-hero',
        ref: 'headerWrapper',
        style: {} },
      React.createElement(
        'header',
        { className: 'entity-header' },
        React.createElement(
          'div',
          { className: 'inner-wrapper' },
          React.createElement(
            'div',
            { className: 'eight-col no-margin-bottom' },
            React.createElement('img', { src: 'data:image/gif;base64,', alt: 'django',
              width: '96', className: 'entity-header__icon' }),
            React.createElement(
              'h1',
              {
                className: 'entity-header__title',
                itemProp: 'name',
                ref: 'entityHeaderTitle' },
              'django',
              ' ',
              React.createElement(
                'span',
                { className: 'entity-header__version' },
                '#',
                123
              )
            ),
            React.createElement(
              'ul',
              { className: 'bullets inline entity-header__properties' },
              React.createElement(
                'li',
                { className: 'entity-header__by' },
                'By\xA0',
                React.createElement(
                  'span',
                  { className: 'link', onClick: instance._onOwnerClick },
                  'test-owner'
                )
              ),
              React.createElement(
                'li',
                { className: 'entity-header__series' },
                React.createElement(
                  'span',
                  { className: 'link',
                    onClick: instance._onLastRevisionClick },
                  'Latest revision (42)'
                )
              ),
              [React.createElement(
                'li',
                { key: 'trusty', className: 'entity-header__series' },
                'trusty'
              )],
              undefined,
              React.createElement(
                'li',
                { key: 'Stable, Candidate',
                  className: 'entity-header__channels' },
                'Stable, Candidate'
              )
            ),
            React.createElement(
              'ul',
              { className: 'entity-header__social-list' },
              React.createElement(
                'li',
                null,
                React.createElement(
                  'a',
                  { id: 'item-twitter',
                    target: '_blank',
                    href: 'https://twitter.com/intent/tweet?text=django%20' + 'charm&via=ubuntu_cloud&url=https%3A%2F%2Fjujucharms' + '.com%2Fdjango%2Ftrusty%2F' },
                  React.createElement(juju.components.SvgIcon, {
                    name: 'icon-social-twitter',
                    size: '36' })
                )
              ),
              React.createElement(
                'li',
                null,
                React.createElement(
                  'a',
                  { id: 'item-googleplus',
                    target: '_blank',
                    href: 'https://plus.google.com/share?url=https%3A%2F%2F' + 'jujucharms.com%2Fdjango%2Ftrusty%2F' },
                  React.createElement(juju.components.SvgIcon, {
                    name: 'icon-social-google',
                    size: '36' })
                )
              )
            )
          ),
          React.createElement(
            'div',
            { className: 'entity-header__right four-col last-col no-margin-bottom' },
            undefined,
            React.createElement(juju.components.CopyToClipboard, {
              value: 'juju deploy cs:django' }),
            React.createElement(juju.components.GenericButton, {
              ref: 'deployAction',
              action: instance._handleDeployClick,
              disabled: false,
              type: 'positive',
              title: 'Add to model' })
          )
        )
      )
    );
    expect(output).toEqualJSX(expectedOutput);
  });

  it('renders an old entity properly', function () {
    mockEntity.set('revision_id', 122);
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.EntityHeader, {
      acl: acl,
      addNotification: sinon.stub(),
      deployService: sinon.spy(),
      changeState: sinon.spy(),
      entityModel: mockEntity,
      getBundleYAML: sinon.stub(),
      getModelName: sinon.stub(),
      hasPlans: false,
      importBundleYAML: sinon.stub(),
      pluralize: sinon.stub(),
      scrollPosition: 0,
      urllib: urllib
    }), true);
    const output = renderer.getRenderOutput();
    assert.deepEqual(output.props.children.props.children.props.children[0].props.children[1].props.children[2].props.children[1], 122);
  });

  it('can display plans', function () {
    const plans = [{ url: 'test' }];
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.EntityHeader, {
      acl: acl,
      addNotification: sinon.stub(),
      deployService: sinon.spy(),
      changeState: sinon.spy(),
      entityModel: mockEntity,
      getModelName: sinon.stub(),
      getBundleYAML: sinon.stub(),
      hasPlans: true,
      plans: plans,
      importBundleYAML: sinon.stub(),
      pluralize: sinon.stub(),
      scrollPosition: 0,
      urllib: urllib
    }), true);
    const output = renderer.getRenderOutput();
    const expectedOutput = React.createElement(
      'select',
      { className: 'entity-header__select',
        ref: 'plan' },
      React.createElement(
        'option',
        { key: 'default' },
        'Choose a plan'
      ),
      [React.createElement(
        'option',
        { key: 'test0',
          value: 'test' },
        'test'
      )]
    );
    expect(output.props.children.props.children.props.children[1].props.children[0]).toEqualJSX(expectedOutput);
  });

  it('displays correctly when loading plans', function () {
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.EntityHeader, {
      acl: acl,
      addNotification: sinon.stub(),
      deployService: sinon.spy(),
      changeState: sinon.spy(),
      entityModel: mockEntity,
      getBundleYAML: sinon.stub(),
      getModelName: sinon.stub(),
      hasPlans: true,
      plans: null,
      importBundleYAML: sinon.stub(),
      pluralize: sinon.stub(),
      scrollPosition: 0,
      urllib: urllib
    }), true);
    const output = renderer.getRenderOutput();
    const expectedOutput = React.createElement(
      'select',
      { className: 'entity-header__select',
        ref: 'plan' },
      React.createElement(
        'option',
        { key: 'default' },
        'Loading plans...'
      ),
      null
    );
    expect(output.props.children.props.children.props.children[1].props.children[0]).toEqualJSX(expectedOutput);
  });

  it('displays correctly when there are no plans', function () {
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.EntityHeader, {
      acl: acl,
      addNotification: sinon.stub(),
      deployService: sinon.spy(),
      changeState: sinon.spy(),
      entityModel: mockEntity,
      getBundleYAML: sinon.stub(),
      getModelName: sinon.stub(),
      hasPlans: true,
      plans: [],
      importBundleYAML: sinon.stub(),
      pluralize: sinon.stub(),
      scrollPosition: 0,
      urllib: urllib
    }), true);
    const output = renderer.getRenderOutput();
    assert.deepEqual(output.props.children.props.children.props.children[1].props.children[0], undefined);
  });

  it('displays the counts for a bundle', function () {
    const pluralize = sinon.stub();
    pluralize.withArgs('application').returns('applications');
    pluralize.withArgs('machine').returns('machines');
    pluralize.withArgs('unit').returns('units');
    const entity = jsTestUtils.makeEntity(true);
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.EntityHeader, {
      acl: acl,
      addNotification: sinon.stub(),
      deployService: sinon.spy(),
      changeState: sinon.spy(),
      entityModel: entity,
      getBundleYAML: sinon.stub(),
      getModelName: sinon.stub(),
      hasPlans: false,
      importBundleYAML: sinon.stub(),
      pluralize: pluralize,
      scrollPosition: 0,
      urllib: urllib
    }), true);
    const output = renderer.getRenderOutput();
    const expectedOutput = React.createElement(
      'ul',
      { className: 'bullets inline entity-header__properties' },
      React.createElement(
        'li',
        { className: 'entity-header__counts' },
        3,
        ' ',
        'applications',
        ', \xA0',
        2,
        ' ',
        'machines',
        ', \xA0',
        5,
        ' ',
        'units'
      )
    );
    expect(output.props.children.props.children.props.children[0].props.children[3]).toEqualJSX(expectedOutput);
  });

  it('can mark charms as subordinates', function () {
    const entity = jsTestUtils.makeEntity(false, { is_subordinate: true });
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.EntityHeader, {
      acl: acl,
      addNotification: sinon.stub(),
      deployService: sinon.spy(),
      changeState: sinon.spy(),
      entityModel: entity,
      getBundleYAML: sinon.stub(),
      getModelName: sinon.stub(),
      hasPlans: false,
      importBundleYAML: sinon.stub(),
      pluralize: sinon.stub(),
      scrollPosition: 0,
      urllib: urllib
    }), true);
    const output = renderer.getRenderOutput();
    const expected = React.createElement(
      'li',
      { className: 'entity-header__subordinate' },
      'Subordinate',
      React.createElement(
        'a',
        { href: 'https://jujucharms.com/docs/stable/' + 'authors-subordinate-applications',
          target: '_blank' },
        React.createElement(juju.components.SvgIcon, {
          name: 'help_16',
          size: '16' })
      )
    );
    expect(output.props.children.props.children.props.children[0].props.children[2].props.children[1]).toEqualJSX(expected);
  });

  it('displays an add to model button', function () {
    const output = testUtils.renderIntoDocument(React.createElement(juju.components.EntityHeader, {
      acl: acl,
      addNotification: sinon.stub(),
      entityModel: mockEntity,
      changeState: sinon.spy(),
      deployService: sinon.spy(),
      getBundleYAML: sinon.stub(),
      getModelName: sinon.stub(),
      hasPlans: false,
      importBundleYAML: sinon.stub(),
      pluralize: sinon.stub(),
      scrollPosition: 0,
      urllib: urllib
    }));
    const deployAction = output.refs.deployAction;
    assert.equal(deployAction.props.type, 'positive');
    assert.equal(deployAction.props.title, 'Add to model');
  });

  it('displays the model name in the add to model button if provided', () => {
    const output = testUtils.renderIntoDocument(React.createElement(juju.components.EntityHeader, {
      acl: acl,
      addNotification: sinon.stub(),
      entityModel: mockEntity,
      changeState: sinon.spy(),
      deployService: sinon.spy(),
      getBundleYAML: sinon.stub(),
      getModelName: sinon.stub().returns('porkchop'),
      hasPlans: false,
      importBundleYAML: sinon.stub(),
      pluralize: sinon.stub(),
      scrollPosition: 0,
      urllib: urllib
    }));
    const deployAction = output.refs.deployAction;
    assert.equal(deployAction.props.type, 'positive');
    assert.equal(deployAction.props.title, 'Add to porkchop');
  });

  it('displays an unsupported message for multi-series charms', function () {
    mockEntity.set('series', undefined);
    const output = testUtils.renderIntoDocument(React.createElement(juju.components.EntityHeader, {
      acl: acl,
      addNotification: sinon.stub(),
      entityModel: mockEntity,
      changeState: sinon.spy(),
      deployService: sinon.spy(),
      getBundleYAML: sinon.stub(),
      getModelName: sinon.stub(),
      hasPlans: false,
      importBundleYAML: sinon.stub(),
      pluralize: sinon.stub(),
      scrollPosition: 0,
      urllib: urllib
    }));
    const textContent = output.refs.deployAction.innerText;
    assert.equal(textContent, 'This type of charm can only be deployed from ' + 'the command line.');
  });

  it('adds a charm when the add button is clicked', function () {
    const deployService = sinon.stub();
    const changeState = sinon.stub();
    const importBundleYAML = sinon.stub();
    const getBundleYAML = sinon.stub();
    const output = testUtils.renderIntoDocument(React.createElement(juju.components.EntityHeader, {
      acl: acl,
      addNotification: sinon.stub(),
      importBundleYAML: importBundleYAML,
      getBundleYAML: getBundleYAML,
      getModelName: sinon.stub(),
      hasPlans: false,
      deployService: deployService,
      changeState: changeState,
      entityModel: mockEntity,
      pluralize: sinon.stub(),
      scrollPosition: 0,
      urllib: urllib
    }));
    const deployAction = output.refs.deployAction;
    // Simulate a click.
    deployAction.props.action();
    assert.equal(deployService.callCount, 1);
    assert.equal(deployService.args[0][0], mockEntity);
  });

  it('can include a plan when deploying a charm', function () {
    const deployService = sinon.stub();
    const changeState = sinon.stub();
    const importBundleYAML = sinon.stub();
    const getBundleYAML = sinon.stub();
    const plans = [{ url: 'test-plan' }];
    const output = testUtils.renderIntoDocument(React.createElement(juju.components.EntityHeader, {
      acl: acl,
      addNotification: sinon.stub(),
      importBundleYAML: importBundleYAML,
      getBundleYAML: getBundleYAML,
      getModelName: sinon.stub(),
      hasPlans: true,
      deployService: deployService,
      changeState: changeState,
      entityModel: mockEntity,
      plans: plans,
      pluralize: sinon.stub(),
      scrollPosition: 0,
      urllib: urllib
    }));
    const refs = output.refs;
    const deployAction = refs.deployAction;
    // Change the select value to a plan.
    refs.plan.value = 'test-plan';
    // Simulate a click.
    deployAction.props.action();
    assert.equal(deployService.callCount, 1);
    assert.deepEqual(deployService.args[0][2], plans);
    assert.deepEqual(deployService.args[0][3], plans[0]);
  });

  it('can set the plans when deploying a charm without selecing one', () => {
    const deployService = sinon.stub();
    const changeState = sinon.stub();
    const importBundleYAML = sinon.stub();
    const getBundleYAML = sinon.stub();
    const plans = [{ url: 'test-plan' }];
    const output = testUtils.renderIntoDocument(React.createElement(juju.components.EntityHeader, {
      acl: acl,
      addNotification: sinon.stub(),
      importBundleYAML: importBundleYAML,
      getBundleYAML: getBundleYAML,
      getModelName: sinon.stub(),
      hasPlans: true,
      deployService: deployService,
      changeState: changeState,
      entityModel: mockEntity,
      plans: plans,
      pluralize: sinon.stub(),
      scrollPosition: 0,
      urllib: urllib
    }));
    const refs = output.refs;
    const deployAction = refs.deployAction;
    // Simulate a click.
    deployAction.props.action();
    assert.equal(deployService.callCount, 1);
    assert.deepEqual(deployService.args[0][2], plans);
    assert.isUndefined(deployService.args[0][3]);
  });

  it('adds a bundle when the add button is clicked', function () {
    const deployService = sinon.stub();
    const changeState = sinon.stub();
    const getBundleYAML = sinon.stub().callsArgWith(1, null, 'mock yaml');
    const importBundleYAML = sinon.stub();
    const entity = jsTestUtils.makeEntity(true);
    const output = testUtils.renderIntoDocument(React.createElement(juju.components.EntityHeader, {
      acl: acl,
      addNotification: sinon.stub(),
      importBundleYAML: importBundleYAML,
      getBundleYAML: getBundleYAML,
      getModelName: sinon.stub(),
      hasPlans: false,
      deployService: deployService,
      changeState: changeState,
      entityModel: entity,
      pluralize: sinon.stub(),
      scrollPosition: 0,
      urllib: urllib
    }));
    const deployAction = output.refs.deployAction;
    // Simulate a click.
    deployAction.props.action();
    assert.equal(getBundleYAML.callCount, 1);
    assert.equal(getBundleYAML.args[0][0], 'django-cluster');
    assert.equal(importBundleYAML.callCount, 1);
    assert.deepEqual(importBundleYAML.args[0][0], 'mock yaml');
  });

  it('displays a notification if there is a bundle deploy error', function () {
    const deployService = sinon.stub();
    const changeState = sinon.stub();
    const getBundleYAML = sinon.stub().callsArgWith(1, 'error');
    const importBundleYAML = sinon.stub();
    const addNotification = sinon.stub();
    const entity = jsTestUtils.makeEntity(true);
    const output = testUtils.renderIntoDocument(React.createElement(juju.components.EntityHeader, {
      acl: acl,
      importBundleYAML: importBundleYAML,
      getBundleYAML: getBundleYAML,
      getModelName: sinon.stub(),
      hasPlans: false,
      deployService: deployService,
      changeState: changeState,
      entityModel: entity,
      addNotification: addNotification,
      pluralize: sinon.stub(),
      scrollPosition: 0,
      urllib: urllib
    }));
    const deployAction = output.refs.deployAction;
    // Simulate a click.
    deployAction.props.action();
    assert.equal(addNotification.callCount, 1);
    assert.deepEqual(addNotification.args[0][0].title, 'Bundle failed to deploy');
  });

  it('can display as sticky', function () {
    const deployService = sinon.stub();
    const changeState = sinon.stub();
    const getBundleYAML = sinon.stub().callsArgWith(1, 'error');
    const importBundleYAML = sinon.stub();
    const addNotification = sinon.stub();
    const entity = jsTestUtils.makeEntity(true);
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.EntityHeader, {
      acl: acl,
      importBundleYAML: importBundleYAML,
      getBundleYAML: getBundleYAML,
      getModelName: sinon.stub(),
      hasPlans: false,
      deployService: deployService,
      changeState: changeState,
      entityModel: entity,
      addNotification: addNotification,
      pluralize: sinon.stub(),
      scrollPosition: 100,
      urllib: urllib
    }), true);
    const instance = renderer.getMountedInstance();
    instance.refs = {
      headerWrapper: {
        clientHeight: 99
      }
    };
    instance.componentDidMount();
    const output = renderer.getRenderOutput();
    const expectedOutput = React.createElement(
      'div',
      { className: 'row-hero',
        ref: 'headerWrapper',
        style: { height: '99px' } },
      React.createElement(
        'header',
        { className: 'entity-header entity-header--sticky' },
        output.props.children.props.children
      )
    );
    expect(output).toEqualJSX(expectedOutput);
  });

  it('goes to the profile page when the owner is clicked', function () {
    const changeState = sinon.stub();
    const evt = {
      preventDefault: sinon.stub().withArgs(),
      stopPropagation: sinon.stub().withArgs()
    };
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.EntityHeader, {
      acl: acl,
      addNotification: sinon.stub(),
      entityModel: mockEntity,
      changeState: changeState,
      deployService: sinon.spy(),
      getBundleYAML: sinon.stub(),
      getModelName: sinon.stub(),
      hasPlans: false,
      importBundleYAML: sinon.stub(),
      pluralize: sinon.stub(),
      scrollPosition: 0,
      urllib: urllib
    }), true);
    const instance = renderer.getMountedInstance();
    instance._onOwnerClick(evt);
    assert.strictEqual(evt.preventDefault.callCount, 1, 'preventDefault');
    assert.strictEqual(evt.stopPropagation.callCount, 1, 'stopPropagation');
    assert.strictEqual(changeState.callCount, 1, 'changeState');
    const args = changeState.args[0];
    assert.strictEqual(args.length, 1, 'changeState args');
    assert.deepEqual(args[0], {
      hash: null,
      store: null,
      profile: 'test-owner'
    });
  });

  it('goes to the last revision when present', function () {
    const changeState = sinon.stub();
    const evt = {
      preventDefault: sinon.stub().withArgs(),
      stopPropagation: sinon.stub().withArgs()
    };
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.EntityHeader, {
      acl: acl,
      addNotification: sinon.stub(),
      entityModel: mockEntity,
      changeState: changeState,
      deployService: sinon.spy(),
      getBundleYAML: sinon.stub(),
      getModelName: sinon.stub(),
      hasPlans: false,
      importBundleYAML: sinon.stub(),
      pluralize: sinon.stub(),
      scrollPosition: 0,
      urllib: urllib
    }), true);
    const instance = renderer.getMountedInstance();
    instance._onLastRevisionClick(evt);
    assert.strictEqual(evt.preventDefault.callCount, 1, 'preventDefault');
    assert.strictEqual(evt.stopPropagation.callCount, 1, 'stopPropagation');
    assert.strictEqual(changeState.callCount, 1, 'changeState');
    const args = changeState.args[0];
    assert.strictEqual(args.length, 1, 'changeState args');
    assert.deepEqual(args[0], { store: 'u/who/django/42' });
  });

  it('can disable the deploy button when read only', function () {
    acl.isReadOnly = sinon.stub().returns(true);
    const renderer = jsTestUtils.shallowRender(React.createElement(juju.components.EntityHeader, {
      acl: acl,
      addNotification: sinon.stub(),
      deployService: sinon.spy(),
      changeState: sinon.spy(),
      entityModel: mockEntity,
      getBundleYAML: sinon.stub(),
      getModelName: sinon.stub(),
      hasPlans: false,
      importBundleYAML: sinon.stub(),
      pluralize: sinon.stub(),
      scrollPosition: 0,
      urllib: urllib
    }), true);
    const instance = renderer.getMountedInstance();
    const output = renderer.getRenderOutput();
    const expectedOutput = React.createElement(juju.components.GenericButton, {
      ref: 'deployAction',
      action: instance._handleDeployClick,
      disabled: true,
      type: 'positive',
      title: 'Add to model' });
    expect(output.props.children.props.children.props.children[1].props.children[2]).toEqualJSX(expectedOutput);
  });
});